<?php
/** 
*
* @package phpBB3
* @version $Id: smartfeed.php, v 2.0.5 Production 2008/05/18 mark@phpbbservices.com Exp $
* @copyright (c) Mark D. Hamill (mhamill@computer.org, http://phpbbservices.com) 
* @license http://opensource.org/licenses/gpl-license.php GNU Public License 
*
*/

/**
* @ignore
*/

// Written by Mark D. Hamill, mhamill@computer.org, http://phpbbservices.com
// This software is designed to work with phpBB Version 3.0.2

// This is the program that creates the RSS or Atom compliant newsfeed that will also include protected
// forums. An enhanced and slightly modified version of the the FeedCreator class, v1.8.0-dev was used 
// to generate the various feeds. The modified version can be found at http://feedcreator.org/.

// A companion program should be run first: smartfeed_url.php. This creates a URL that is
// used to invoke the feed. Use the URL created by this program in your newsreader. Do NOT try to run 
// this program without arguments!

// Set up the phpBB environment we need 
define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);
require_once($phpbb_root_path . 'includes/constants_smartfeed.' . $phpEx);

// We need to start a session to take advantage of the benefits it gives, like access to common function, sessions, etc.
$user->session_begin();
$auth->acl($user->data);

// Note: one limitation of adding the smartfeed language file is that it inherits the guest default language 
// unless the user is authenticated automatically, such as via cookie authentication. It behooves the board administrator
// to make sure he has a language specific version of all Smartfeed language files for each language allowed on the board. 
$user->setup('mods/smartfeed'); 

// Set up some variables
$registered_user = false;
$allowed_user_types = array(USER_NORMAL, USER_FOUNDER); // Allowed users are Normal and Founder. Others (Inactive, Ignore) can only get a public feed.
$board_url = generate_board_url() . '/';
$true_false_array = array(0, 1);

// ------ First, report errors for any inconsistent URL errors ------ //

// Arguments are required when running smartfeed.php
if (!isset($_SERVER['QUERY_STRING']) || (strlen($_SERVER['QUERY_STRING']) == 0))
{
	$feed_type = $config['sf_rss_20_value'];
	handle_error('SMARTFEED_NO_ARGUMENTS');
}

// Validate the feed type. Atom 1.0, RSS 2.0 and RSS 1.0 are the only feed types allowed.
$feed_type = request_var('feed_type','NONE');
if (($feed_type == 'NONE') || !(($feed_type == $config['sf_atom_10_value']) || 
	($feed_type == $config['sf_rss_20_value']) || 
	($feed_type == $config['sf_rss_10_value'])))
{
	// Bad feed_type, trigger error
	handle_error('SMARTFEED_FEED_TYPE_ERROR');
}

// If the board is currently disabled, feeds should also be disabled, don't ya think?
if ($config['board_disable'])
{
	handle_error('SMARTFEED_BOARD_DISABLED');
}

// Get the user id. A public user won't be identified as a user in the URL.
$user_id = request_var('u', ANONYMOUS);

// Get the encrypted password. When decrypted it is still encoded md5 as it should also be in the database
$encrypted_pswd = request_var('e', 'NONE');

// Determine if this is a public request. If so only public forums will be shown.
if ($user_id != ANONYMOUS && $encrypted_pswd != 'NONE')
{
	// Feed privileges are dependent upon the auth_method. This code makes this program consistent with smartfeed_url.php
	switch ($config['auth_method'])
	{
		case 'db':
			$registered_user = true;
			break;
		case 'apache':
			if ($config['sf_apache_htaccess_enabled'])
			{
				$registered_user = true;
			}
			else
			{
				handle_error('SMARTFEED_APACHE_AUTHENTICATION_WARNING_REG');
			}
			break;
		case 'ldap':
		default:
			$user_id = ANONYMOUS;
			$encrypted_pswd = 'NONE';
			break;
	}
}
else if (!(($user_id == ANONYMOUS) && ($encrypted_pswd == 'NONE')))
{
	// Logically if only the u or the e parameter is present, the URL is inconsisent, so generate an error.
	if ($user_id == ANONYMOUS)
	{
		handle_error('SMARTFEED_NO_U_ARGUMENT');
	}
	if ($encrypted_pswd == 'NONE')
	{
		handle_error('SMARTFEED_NO_E_ARGUMENT');
	}
}

// Get the limit parameter. It limits the size of the newsfeed to a point in time from the present, either a day/hour/minute interval, no limit
// or the time since the user's last visit. It should always exist. If it doesn't exist or the value passed is invalid, trigger an error.
// Please note that for public users no message older than SMARTFEED_DEFAULT_FETCH_TIME_LIMIT can actually be retrieved to avoid huge 
// newsfeeds and excessive strain on the database.
$time_limit = request_var('limit', '', true);
if ($registered_user && (!(in_array($time_limit, $smartfeed['SMARTFEED_TIME_LIMIT_REGISTERED']))))
{
	handle_error('SMARTFEED_LIMIT_FORMAT_ERROR');
}
if (!$registered_user && (!(in_array($time_limit, $smartfeed['SMARTFEED_TIME_LIMIT_UNREGISTERED']))))
{
	handle_error('SMARTFEED_LIMIT_FORMAT_ERROR');
}

// Validate the sort_by parameter, which should always be present
$sort_by = request_var('sort_by','NONE', true);
if (($sort_by == 'NONE') || (!(in_array($sort_by, $smartfeed['SMARTFEED_SORT_BY_VALUES']))))
{
	// Bad sort_bye, trigger error
	handle_error('SMARTFEED_SORT_BY_ERROR');
}

// Validate the firstpostonly parameter.
$first_post_only = request_var('firstpostonly','');
if (!is_numeric($first_post_only) && $first_post_only != '')
{
	handle_error('SMARTFEED_FIRST_POST_ONLY_ERROR');
}
$first_post_only = intval($first_post_only);
if ((!in_array(intval($first_post_only),$true_false_array)) || !is_numeric($first_post_only) )
{
	// Bad firstpostonly value, trigger error
	handle_error('SMARTFEED_FIRST_POST_ONLY_ERROR');
}

// Check for count limit parameter. It is not required, but if present should be a positive number only. The value must
// be less than or equal to $config['sf_max_items']. But if $config['sf_max_items'] == 0 then any positive whole number is allowed.
// If not present the count limit is $config['sf_max_items'] if positive, or unlimited if this value is zero.
$count_limit = request_var('count_limit','');
if (!is_numeric($count_limit) && $count_limit != '')
{
	handle_error('SMARTFEED_COUNT_LIMIT_FORMAT_ERROR');
}
$count_limit = intval($count_limit);
if ($count_limit < 0)
{
	handle_error('SMARTFEED_COUNT_LIMIT_ERROR');
}
if (($config['sf_max_items'] > 0) && ($count_limit > $config['sf_max_items']))
{
	handle_error('SMARTFEED_COUNT_LIMIT_MAX_ERROR');
}
if (($config['sf_max_items'] > 0) && ($count_limit == 0))
{
	$count_limit = $config['sf_max_items'];
}

// Validate the maximum size requested for the number of words in a post
$max_word_size = request_var('max_word_size','');
if (!is_numeric($max_word_size) && $max_word_size != '')
{
	handle_error('SMARTFEED_MAX_WORD_SIZE_ERROR');
}
$max_word_size = intval($max_word_size);
if ($max_word_size < 0)
{
	// Bad max_word_size, trigger error
	handle_error('SMARTFEED_MAX_WORD_SIZE_ERROR');
}
	
// Validate the minimum size requested for the number of words in a post
$min_word_size = request_var('min_word_size','');
if (!is_numeric($min_word_size) && $min_word_size != '')
{
	handle_error('SMARTFEED_MAX_WORD_SIZE_ERROR');
}
$min_word_size = intval($min_word_size);
if ($min_word_size < 0)
{
	handle_error('SMARTFEED_MIN_WORD_SIZE_ERROR');
}
	
// Validate the feed_style parameter.
$feed_style = request_var('feed_style','', true);
if (($sort_by == '') || (!(in_array($feed_style, $smartfeed['SMARTFEED_FEED_STYLE_VALUES']))))
{
	handle_error('SMARTFEED_STYLE_ERROR');
}

if ($registered_user)
{
	// Validate the removemine parameter, if present
	$remove_mine = request_var('removemine','');
	if (!is_numeric($remove_mine) && $remove_mine != '')
	{
		handle_error('SMARTFEED_REMOVE_MINE_ERROR');
	}
	$remove_mine = intval($remove_mine);
	if (!in_array($remove_mine,$true_false_array))
	{
		// Bad sort_bye, trigger error
		handle_error('SMARTFEED_REMOVE_MINE_ERROR');
	}

	// Validate the private messages switch
	$show_pms = request_var('pms','');
	if (!is_numeric($show_pms) && $show_pms != '')
	{
		handle_error('SMARTFEED_BAD_PMS_VALUE');
	}
	$show_pms = intval($show_pms);
	if (!in_array($show_pms,$true_false_array))
	{
		// Bad parameter value for private messages
		handle_error('SMARTFEED_BAD_PMS_VALUE');
	}

	// Validate the bookmarks switch
	$bookmarks = request_var('bookmarks','');
	if (!is_numeric($bookmarks) && $bookmarks != '')
	{
		handle_error('SMARTFEED_BAD_BOOKMARKS_VALUE');
	}
	$bookmarks = intval($bookmarks);
	if (!in_array($bookmarks,$true_false_array))
	{
		// Bad parameter value for private messages
		handle_error('SMARTFEED_BAD_BOOKMARKS_VALUE');
	}

	// Validate the filter_foes parameter.
	$filter_foes = request_var('filter_foes','');
	if (!is_numeric($filter_foes) && $filter_foes != '')
	{
		handle_error('SMARTFEED_FILTER_FOES_ERROR');
	}
	$filter_foes = intval($filter_foes);
	if (!in_array($filter_foes,$true_false_array))
	{
		handle_error('SMARTFEED_FILTER_FOES_ERROR');
	}
	
	// Validate the lastvisit parameter.
	$lastvisit = request_var('lastvisit','');
	if (!is_numeric($lastvisit) && $lastvisit != '')
	{
		handle_error('SMARTFEED_LASTVISIT_ERROR');
	}
	$lastvisit = intval($lastvisit);
	if (!in_array($lastvisit,$true_false_array))
	{
		handle_error('SMARTFEED_LASTVISIT_ERROR');
	}

}

// ------ Second, authenticate the user ------ //

// We need the user_permissions column even for Anonymous, so we make this database call even if they cannot log in
$sql = 'SELECT user_id, user_password, user_permissions, user_smartfeed_key, user_topic_sortby_type, user_topic_sortby_dir, 
			user_post_sortby_type, user_post_sortby_dir, user_lastvisit, user_type
		FROM ' . USERS_TABLE . " WHERE user_id = $user_id";
if ($user_id != ANONYMOUS)
{
	$sql .= ' AND ' . $db->sql_in_set('user_type', $allowed_user_types); // Robots and inactive members are not allowed to get feeds
}

$result = $db->sql_query($sql);
$rowset = $db->sql_fetchrowset($result);

if (sizeof($rowset) == 0)
{
	handle_error('SMARTFEED_USER_ID_DOES_NOT_EXIST');
}

// Make sure user_id exists in database and has normal or founder status
$row = reset($rowset);

// Save the user variables, although only the first is unneeded for guests.
$user_permissions = $row['user_permissions'];
$user_smartfeed_key = $row['user_smartfeed_key'];
$user_topic_sortby_type = $row['user_topic_sortby_type']; 
$user_topic_sortby_dir = $row['user_topic_sortby_dir']; 
$user_post_sortby_type = $row['user_post_sortby_type']; 
$user_post_sortby_dir = $row['user_post_sortby_dir'];

// These other variables apply only to registered users
$user_password = $row['user_password'];
$user_lastvisit = $row['user_lastvisit'];

$db->sql_freeresult($result); // Query be gone!

if ($registered_user)
{
	
	// Because base64_encode was used instead of urlencode we get a relatively compact e parameter. However, this has one small
	// drawback: + signs in e parameter are translated to a space character when read. Consequently we have to put them back in.
	$encrypted_pswd = str_replace(' ','+',$encrypted_pswd);
	
	// Decrypt password using the user_smartfeed_key column in the phpbb_users table. This should have gotten created 
	// the first time smartfeed_url.php was run by this user. Note the encoded password is typically md5. There should not be
	// a clear text password in the database.
	if (strlen($user_smartfeed_key) > 0)
	{
		$encoded_pswd = decrypt($encrypted_pswd, $user_smartfeed_key);
	}
	else
	{
		// If the $user_smartfeed_key is null, password cannot be decrypted. This avoids PHP Notice statements in TripleDES.php
		handle_error('SMARTFEED_BAD_PASSWORD_ERROR');
	}
				
	// If IP Authentication was enabled, the encoded password is to the left of the ~ and the IP to the right of the ~
	$tilde = strpos($encoded_pswd, '~');
	if (($tilde == 0) && ($config['sf_require_ip_authentication'] == '1'))
	{
		handle_error('SMARTFEED_IP_AUTH_ERROR');
	}
	
	if ($tilde > 0)
	{
		// Since a tilde is present, authenticate the client IP by comparing it with the IP embedded in the "e" parameter
		$auth_ip = substr($encoded_pswd,$tilde+1);
		$encoded_pswd = substr($encoded_pswd,0,$tilde);
		$client_ip_parts = explode('.', $_SERVER['REMOTE_ADDR']);
		$source_ip_parts = explode('.', $auth_ip);
		// Show error message if requested from incorrect range of IP addresses
		if (!(($client_ip_parts[0] == $source_ip_parts[0]) && ($client_ip_parts[1] == $source_ip_parts[1]) && ($client_ip_parts[2] == $source_ip_parts[2])))
		{
			handle_error('SMARTFEED_IP_AUTH_ERROR');
		}
	}

	if ($encoded_pswd != $user_password)
	{
		handle_error('SMARTFEED_BAD_PASSWORD_ERROR');
	} 

}

// If requested to fetch new posts since the user's last visit, we need to get the user_lastvisit date from the database. Of course this makes
// no sense if the user is Anonymous. Cookie logic in the phpBB 2 version has been removed to simplify things. Most newsreaders won't store cookies
// anyway.

switch($config['sf_default_fetch_time_limit'])
{
	case 0: // Admin has set no limit to how far back posts can be fetched
		switch ($time_limit)
		{
			case SMARTFEED_SINCE_LAST_VISIT:
				$date_limit = $user_lastvisit;
				$date_limit_sql = ' AND p.post_time > ' . $date_limit;
				break;
			case SMARTFEED_NO_LIMIT_VALUE:
				$date_limit = 0;
				$date_limit_sql = '';
				break;
			default:
				$date_limit = $smartfeed['SMARTFEED_TIME_LIMIT_SECONDS'][$time_limit];
				$date_limit_sql = ' AND p.post_time > ' . $date_limit;
				break;
		}
		break;
		
	default: // Admin has set a limit to how far back posts can be fetched
		switch ($time_limit)
		{
			case SMARTFEED_SINCE_LAST_VISIT:
				$date_limit = max($user_lastvisit, time() - ($config['sf_default_fetch_time_limit'] * 3600));
				$date_limit_sql = ' AND p.post_time > ' . $date_limit;
				break;
			case SMARTFEED_NO_LIMIT_VALUE:
				$date_limit = $config['sf_default_fetch_time_limit'] * 3600;
				$date_limit_sql = ' AND p.post_time > ' . $date_limit;
				break;
			default:
				$date_limit = max($smartfeed['SMARTFEED_TIME_LIMIT_SECONDS'][$time_limit], time() - ($config['sf_default_fetch_time_limit'] * 3600));
				$date_limit_sql = ' AND p.post_time > ' . $date_limit;
				break;
		}
		break;
}

// ------ Third, create a list of forums that will be fetched ------ //

$fetched_forum_str = '';

if ($registered_user && $bookmarks)
{

	// When selecting bookmarked topics only, we can safely ignore the logic constraining the user to read only 
	// from certain forums. Instead we will create the SQL to get the bookmarked topics, if any, hijacking the 
	// $fetched_forum_str variable since it is convenient
	
	$bookmarked_topics = array();
	$sql = 'SELECT t.topic_id
			FROM ' . USERS_TABLE . ' u, ' . BOOKMARKS_TABLE . ' b, ' . TOPICS_TABLE . " t
			WHERE u.user_id = b.user_id AND b.topic_id = t.topic_id 
				AND t.topic_last_post_time > $date_limit
				AND b.user_id = $user_id";
	$result = $db->sql_query($sql);
	while ($row = $db->sql_fetchrow($result))
	{
		$bookmarked_topics[] = intval($row['topic_id']);
	}
	$db->sql_freeresult($result);
	if (sizeof($bookmarked_topics) > 0)
	{
		$fetched_forums_str = ' AND ' . $db->sql_in_set('t.topic_id', $bookmarked_topics);
	}
	else
	{
		// Logically, if there are no bookmarked topics for this $user_id then there will be nothing in the feed.
		// Send a message to this effect in the feed.
		handle_error('SMARTFEED_NO_BOOKMARKS');
	}

}
else
{

	// Determining forum permissions is more complex under phpBB 3. The essential information is stored in the user_permissions column in the phpbb_users
	// table, typically in a nasty BLOB object. I am sure this is very efficient, but it is confusing to us mod authors! The reason we have to go with this
	// approach is because we don't know if the user's board cookies will be stored on this machine, or if the newsreader is even capable of using cookies. 
	// Most likely phpBB will see this user as "Anonymous" which means the acl_getf function cannot be used. 
	
	// Begin code stolen and tweaked from auth.php
	
	$sql = 'SELECT auth_option, is_global, is_local
			FROM ' . ACL_OPTIONS_TABLE . '
			ORDER BY auth_option_id';
	$result = $db->sql_query($sql);
	
	$local = 0;
	$acl_options = array();
	$acl = array();
	while ($row = $db->sql_fetchrow($result))
	{
		if ($row['is_local'])
		{
			$acl_options['local'][$row['auth_option']] = $local++;
		}
	}
	$db->sql_freeresult($result);
	
	$user_permissions = explode("\n", $user_permissions);
	
	foreach ($user_permissions as $f => $seq)
	{
		if ($seq)
		{
			$i = 0;
	
			if (!isset($acl[$f]))
			{
				$acl[$f] = '';
			}
	
			while ($subseq = substr($seq, $i, 6))
			{
				// We put the original bitstring into the acl array
				$acl[$f] .= str_pad(base_convert($subseq, 36, 2), 31, 0, STR_PAD_LEFT);
				$i += 6;
			}
		}
	}
	
	// End code stolen from auth.php
	
	$f_read_local_index = -1;
	foreach ($acl_options['local'] as $key => $value)
	{
		if ($key == 'f_read')
		{
			$f_read_local_index = $value;
			break;
		}
	}
	
	if ($f_read_local_index == -1)
	{
		handle_error('SMARTFEED_F_READ_ERROR');
	}

	// Create the allowed forums array. These are the forums for which this user has read privileges. He can select
	// ONLY from these forums. If they request they can choose from a smaller subset of these forums.
	
	$allowed_forums = array();
	foreach ($acl as $key => $value)
	{
		if (substr($value, $f_read_local_index, 1) == '1')
		{
			$allowed_forums[] = $key;
		}
	}
	if (sizeof($allowed_forums) == 0)
	{
		// If this user cannot retrieve ANY forums, this suggests that this board is tightly locked down to members only,
		// or every member must belong to a user group or have special forum permissions
		handle_error('SMARTFEED_NO_ACCESSIBLE_FORUMS');
	}
	
	// Get the requested forums. If none are listed, user wants all forums for which they have read access.
	$requested_forums = array();
	$query_string = ( !empty($_SERVER['QUERY_STRING']) ) ? $_SERVER['QUERY_STRING'] : '';
	$params = explode('&', $query_string);
	foreach ($params as $item)
	{
		if (substr($item,0,5) == 'forum')
		{
			$requested_forums[] = substr($item,6);
		}
	}
	
	// To capture global announcements when forums are specified, we have to add the pseudo-forum with a forum_id = 0.
	if (sizeof($requested_forums) > 0)
	{
		$requested_forums[] = '0';
	}
	
	// Sort requested forums by forum_id and ensure there are no duplicates
	asort($requested_forums);
	$requested_forums = array_unique($requested_forums);
	
	// The forums that will be fetched is the array intersection of the requested and allowed forums. This prevents hacking
	// the URL to get feeds a user is not supposed to get. If no forums are specified on the URL field
	// then all forums that this user is authorized to access is assumed.
	
	$fetched_forums = (sizeof($requested_forums) > 0) ? array_intersect($allowed_forums, $requested_forums): $allowed_forums;
	$fetched_forums_str = ' AND ' . $db->sql_in_set('p.forum_id', $fetched_forums);

}

// ------ Fourth, create the snippets of SQL used to return the set of posts ------ //

// Create the SQL stub for the sort order
switch($sort_by)
{
	case SMARTFEED_BOARD:
		$topic_asc_desc = ($user_topic_sortby_dir == 'd') ? 'DESC' : '';
		switch($user_topic_sortby_type)
		{
			case 'a':
				$order_by_sql = "t.topic_first_poster_name $topic_asc_desc, ";
				break;
			case 't':
				$order_by_sql = "t.topic_last_post_time $topic_asc_desc, ";
				break;
			case 'r':
				$order_by_sql = "t.topic_replies $topic_asc_desc, ";
				break;
			case 's':
				$order_by_sql = "t.topic_title $topic_asc_desc, " ; 
				break;
			case 'v':
				$order_by_sql = "t.topic_views $topic_asc_desc, ";
				break;
		}
		$post_asc_desc = ($user_post_sortby_dir == 'd') ? 'DESC' : '';
		switch($user_post_sortby_type)
		{
			case 'a':
				$order_by_sql .= "u.username_clean $post_asc_desc";
				break;
			case 't':
				$order_by_sql .= "p.post_time $post_asc_desc";
				break;
			case 's':
				$order_by_sql .= "p.post_subject $post_asc_desc" ; 
				break;
		}
		break;
	case SMARTFEED_STANDARD:
		$order_by_sql = 'f.left_id, f.right_id, t.topic_last_post_time, p.post_time';
		break;
	case SMARTFEED_STANDARD_DESC:
		$order_by_sql = 'f.left_id, f.right_id, t.topic_last_post_time, p.post_time DESC';
		break;
	case SMARTFEED_POSTDATE:
		$order_by_sql = 'p.post_time';
		break;
	case SMARTFEED_POSTDATE_DESC:
		$order_by_sql = 'p.post_time DESC';
		break;
}

$new_topics_sql = '';
$topics_posts_join_sql = 'p.topic_id = t.topic_id ';

// Create the first_post_only SQL stubs
if ($first_post_only)
{
	$new_topics_sql = " AND t.topic_time > $date_limit ";
	$topics_posts_join_sql = ' t.topic_first_post_id = p.post_id AND t.forum_id = f.forum_id';
}

// Create SQL to remove your posts from the feed
$remove_mine_sql = '';
if ($registered_user && ($remove_mine == 1))
{
	$remove_mine_sql = " AND p.poster_id <> $user_id ";
}

// Create SQL to remove your foes from the feed
$filter_foes_sql = '';
$foes = array();
if ($registered_user && ($filter_foes == 1))
{

	// Fetch your foes
	$sql = 'SELECT zebra_id 
			FROM ' . ZEBRA_TABLE . "
			WHERE user_id = $user_id AND foe = 1";
	$result = $db->sql_query($sql);
	while ($row = $db->sql_fetchrow($result))
	{
		$foes[] = (int) $row['zebra_id'];
	}
	$db->sql_freeresult($result);

	if (sizeof($foes) > 0)
	{
		$filter_foes_sql = ' AND ' . $db->sql_in_set('p.poster_id', $foes, true);
	}
	
}

// At last, construct the SQL to return the relevant posts
			
$sql_array = array(
	'SELECT'	=> 'f.*, t.*, p.*, u.*',

	'FROM'		=> array(
		POSTS_TABLE => 'p',
		USERS_TABLE => 'u',
		TOPICS_TABLE => 't'	),

	'WHERE'		=> "$topics_posts_join_sql
				AND p.poster_id = u.user_id
				$date_limit_sql
				$fetched_forums_str
				$new_topics_sql
				$remove_mine_sql
				$filter_foes_sql
				AND p.post_approved = 1",

	'ORDER_BY'	=> $order_by_sql
);

$sql_array['LEFT_JOIN'] = array(
	array(
		'FROM'	=> array(FORUMS_TABLE => 'f', TOPICS_TABLE => 't'),
		'ON'	=> 't.forum_id = f.forum_id'
	)
);

$sql = $db->sql_build_query('SELECT', $sql_array);

// Now finally, let's fetch the actual posts to be placed in this newsfeed
$result = $db->sql_query_limit($sql, $count_limit); // Execute the SQL to retrieve the relevant posts. Note, if $count_limit is 0 then there is no limit on the rows returned
$rowset = $db->sql_fetchrowset($result); // Get all the posts as a set

if ($registered_user && $show_pms)
{

	// If there are any unread private messages, they are fetched separately and passed as a rowset to publish_feed.
	$pm_sql = 	'SELECT *
				FROM ' . PRIVMSGS_TO_TABLE . ' pt, ' . PRIVMSGS_TABLE . ' pm, ' . USERS_TABLE . " u
				WHERE pt.msg_id = pm.msg_id
					AND pt.author_id = u.user_id
					AND pt.user_id = $user_id
					AND (pm_unread = 1 OR pm_new = 1)";
	$pm_result = $db->sql_query($pm_sql);
	$pm_rowset = $db->sql_fetchrowset($pm_result);

}
else
{
	$pm_result = NULL;
	$pm_rowset = NULL;
}

publish_feed($rowset, $pm_rowset); // Publish the set of rows (posts and private messages) as feed items
$db->sql_freeresult($result);
$db->sql_freeresult($pm_result);

// Reset the user's last visit date on the forum, if so requested
if ($lastvisit)
{
	$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_lastvisit = ' . time() . " 
				WHERE user_id = $user_id";
		
	$result = $db->sql_query($sql);
}

// Session can be killed at this point and is normally preferred so as not to skew online and guest statistics, but not if:
//	- $config['sf_show_sessions'] is true
//	- User is administrator, because they are likely testing and don't want the hassle of logging back in
//	- User just ran smartfeed_url.php, because they are likely testing too

$kill_session = true;
if ($config['sf_show_sessions'])
{
	$kill_session = false; 
}
elseif ($user->data['user_type'] == USER_FOUNDER)
{
	$kill_session = false; 
}
elseif (isset($_SERVER['HTTP_REFERER']) && (strpos($_SERVER['HTTP_REFERER'],'smartfeed_url') === true))
{
	$kill_session = false; 
}

// Kill the session, but only if smartfeed_url.php was not calling this program. Otherwise, the user's phpBB session will end
if ($kill_session)
{
	kill_session();
}

// All done!
exit;

function kill_session ()
{

	// Kill the sessions we consumed. We don't want to use session_kill() because it updates user_lastvisit,
	// which we don't necessarily want to do since the user may want to visit the board later and see
	// messages in the feed as unread.
	
	global $db, $user, $config, $phpEx, $phpbb_root_path;
	
	$sql = 'DELETE FROM ' . SESSIONS_TABLE . "
			WHERE session_id = '" . $db->sql_escape($user->session_id) . "'
				AND session_user_id = " . (int) $user->data['user_id'];
	$db->sql_query($sql);
	
	// Allow connecting logout with external auth method logout
	$method = basename(trim($config['auth_method']));
	include_once($phpbb_root_path . 'includes/auth/auth_' . $method . '.' . $phpEx);

	$method = 'logout_' . $method;
	if (function_exists($method))
	{
		$method($user->data, $new_session);
	}

	return;
}

function handle_error($error_code)
{

	// Publish the error as a newsfeed with a single item in it.
	
	global $user, $encrypted_pswd, $user_id, $board_url, $phpEx, $feed_type, $_SERVER, $config;
	if (!(isset($feed_type)))
	{
		$feed_type = $config['sf_rss_20_value'];
	}
	
	switch ($error_code)
	{
		case 'SMARTFEED_BAD_PASSWORD_ERROR':
			$error_msg = sprintf($user->lang['SMARTFEED_ERROR'][$error_code], $encrypted_pswd, $user_id, $board_url . "smartfeed_url.$phpEx");
			break;
		case 'SMARTFEED_COUNT_LIMIT_MAX_ERROR':
			$error_msg = sprintf($user->lang['SMARTFEED_ERROR'][$error_code], $config['sf_max_items']);
			break;
		default:
			$error_msg = $user->lang['SMARTFEED_ERROR'][$error_code];
			break;
	}
	
	publish_feed($error_msg);
	
	// Kill the session, but only if smartfeed_url.php was not calling this program. Otherwise, the user's phpBB session will end
	if (isset($_SERVER['HTTP_REFERER']) && strpos($_SERVER['HTTP_REFERER'],'smartfeed_url') === false)
	{
		kill_session();
	}
	
	exit;

}

function publish_feed($rowset, $pm_rowset = NULL)
{

	// This function does the grunt work of changing rows in a SQL query into a newsfeed. It also handles
	// any potential errors that may have occurred by placing the error as the sole item of the feed itself. 
	
	global $board_url, $user, $feed_type, $phpEx, $config, $phpbb_root_path, $db, $registered_user, $max_word_size, $min_word_size, $feed_style, $user_id;
	
	require($phpbb_root_path . "includes/feedcreator.class.$phpEx"); 

	// $allowable_tags is used with strip_tags. This removed "dangerous" tags that in the W3C's opinions should not be embedded in a feed,
	// including within a CDATA section. Any tag that is not one of these is stripped providing the feed style is Safe HTML.
	$allowable_tags = '<abbr><accept><accept-charset><accesskey><action><align><alt><axis><border><cellpadding><cellspacing><char><charoff><charset><checked><cite><class><clear><cols><colspan><color><compact><coords><datetime><dir><disabled><enctype><for><frame><headers><height><href><hreflang><hspace><id><ismap><label><lang><longdesc><maxlength><media><method><multiple><name><nohref><noshade><nowrap><prompt><readonly><rel><rev><rows><rowspan><rules><scope><selected><shape><size><span><src><start><summary><tabindex><target><title><type><usemap><valign><value><vspace><width>'; 
   	
	if (is_string($rowset)) // If the first argument is a string rather than an array, some error occurred so we need to publish the error as a newsfeed item
	{
		$error_mode = true;
		$error_msg = $rowset;	// The argument contains the error message
	}
	else 
	{
		// This is the typical case. We are publishing a newsfeed
		$error_mode = false;
	}
		
	// Initialize the feed
	$rss = new UniversalFeedCreator();
	$rss->useCached($feed_type, '', $config['sf_ttl'] * 60);
	$rss->title = html_entity_decode($config['sitename']);
	$rss->description = $config['site_desc'];
	$rss->link = ($feed_type == $config['sf_atom_10_value']) ? $board_url . "smartfeed.$phpEx?" . $_SERVER['QUERY_STRING'] : $board_url . "smartfeed.$phpEx?" . htmlspecialchars($_SERVER['QUERY_STRING']);
	$rss->syndicationURL = $board_url . "smartfeed.$phpEx";
	
	// Some RSS 2.0 tags
	$rss->language = $config['sf_rfc1766_lang'];
	$rss->pubDate = time();
	$rss->ttl = $config['sf_ttl'];
	$rss->copyright = $user->lang['SMARTFEED_COPYRIGHT'];
	$rss->editor = $user->lang['SMARTFEED_EDITOR'];
	$rss->editorEmail = $user->lang['SMARTFEED_EDITOR_EMAIL'];
	$rss->webmaster = $config['sf_webmaster'];
	
	// Put an image in the feed. Typically this will be the phpBB default image
	$image = new FeedImage();
	$image->title = $rss->title; // Enhance to use default logo if not specified
	$image->url = $board_url . 'styles/' . get_default_style() . '/' . $config['sf_feed_image_path']; 
	$image->link = $rss->link;
	$image->description = strip_tags($config['site_desc']);
	$rss->image = $image;
	
	if ($error_mode)
	{
		// Place the error as a feed item
		$item = new FeedItem();
		$item->title = html_entity_decode($user->lang['SMARTFEED_ERROR_TITLE']);
		$item->description = sprintf($user->lang['SMARTFEED_ERROR_INTRODUCTION'], $board_url . "smartfeed_url.$phpEx") . $error_msg;
		$item->author = $config['board_contact'];
		$item->authorEmail = $config['board_email'];
		$item->link = $board_url . "smartfeed_url.$phpEx";
		$item->guid = $item->link;
		$rss->addItem($item);
	}
	else
	{
	
		// If there are any unread private messages, publish them first
		if (is_array($pm_rowset) && (sizeof($pm_rowset) > 0))
		{
		
			foreach ($pm_rowset as $row)
			{

				// This private message goes in the newsfeed
				$username = $row['username']; // Don't need to worry about Anonymous users for private messages, they cannot send them
	
				$item = new FeedItem();
	
				// Create the title for the private message
				$title = $user->lang['PRIVATE_MESSAGE'] . $user->lang['SMARTFEED_DELIMITER'] . $row['message_subject'] . $user->lang['SMARTFEED_DELIMITER'] . $user->lang['FROM'] . ' ' . $username;
				
				$item->title = html_entity_decode(censor_text($title));
				
				// Create a number of item attributes, not all of which are necessarily in the feed (it depends on the feed type)
				$item->date = $row['message_time'];
				$item->pubDate = $item->date;
				$item->link = $board_url . 'ucp.' . $phpEx . '?i=pm&mode=view&f=0&p=' . $row['msg_id'];
				$item->source = $board_url;
				$item->category = $user->lang['PRIVATE_MESSAGE'];
				$item->guid = $item->link;
				$item->descriptionHtmlSyndicated = true;
				// The comment URL is not generated until the message is read, so we have to use the same URL as the view message link
				$item->comments = $board_url . 'ucp.' . $phpEx . '?i=pm&mode=view&f=0&p=' . $row['msg_id'];
				$item->author = $username;
	
				if ($feed_type==$config['sf_rss_20_value']) 
				{
					// RSS 2.0 requires an email field to validate. Use a fake email address (no_email@example.com), set in feed_creator.class.php 
					// if the author field is blank) unless in user profile it says it is okay to show email address.
					$item->authorEmail = (($row['user_allow_viewemail'] == '1') && (!$config['sf_privacy_mode'])) ? $row['user_email'] : '';
				}               
				else
				{
					$item->authorEmail = '';
				}
				
				// To "dress up" the message text with bbCode, images, smilies etc., we need to use generate_text_for_display() function. 
				
				$message = censor_text($row['message_text']);
				
				$user_sig = ( $row['enable_sig'] && ($row['user_sig'] != '') && $config['allow_sig'] && ($config['sf_privacy_mode'] == '0') ) ? $row['user_sig'] : '';
				
				if (($feed_style == SMARTFEED_HTML) || ($feed_style == SMARTFEED_HTMLSAFE))
				{
					$flags = (($row['enable_bbcode']) ? OPTION_FLAG_BBCODE : 0) +
						(($row['enable_smilies']) ? OPTION_FLAG_SMILIES : 0) + 
						(($row['enable_magic_url']) ? OPTION_FLAG_LINKS : 0);
						
					$message = generate_text_for_display($message, $row['bbcode_uid'], $row['bbcode_bitfield'], $flags);
					$message = str_replace('<img src="./', '<img src="' . $board_url, $message); 
		
					if ($user_sig != '')
					{
						$user_sig = generate_text_for_display(censor_text($user_sig), $row['user_sig_bbcode_uid'], $row['user_sig_bbcode_bitfield'], $flags);
					}
		
					$message = ($user_sig != '') ? $message . $user->lang['SMARTFEED_POST_SIGNATURE_DELIMITER'] . $user_sig : $message;
					if ($feed_style == SMARTFEED_HTMLSAFE)
					{
						$message = strip_tags($message, $allowable_tags);
					}
				}
				else
				{
					// Either Compact or Basic Style wanted
					if ($feed_style == SMARTFEED_BASIC)
					{
						$message = ($user_sig != '') ? $message . "\n\n" . $user_sig : $message;
					}
					strip_bbcode($message); 			// Remove the BBCode
					$message = strip_tags($message);	// Gets rid of any embedded HTML
					// Either condense all text or make line feeds explicit
					$message = ($feed_style == SMARTFEED_BASIC) ? nl2br($message) : str_replace("\n", ' ', $message);
				}
				
				// Handle the maximum number of words requested per PM logic
				if ($max_word_size != 0)
				{
					$message = truncate_words($message, intval($max_word_size));
				}
				
				// Add the item (post) to the feed
				$item->description = $message;
				$rss->addItem($item);
				
				// Mark the private message as read
				$sql = 'UPDATE ' . PRIVMSGS_TO_TABLE . "
						SET pm_unread = 0, pm_new = 0  
						WHERE msg_id = " . $row['msg_id'] . " 
							AND user_id = $user_id
							AND author_id = " . $row['author_id'] . " 
							AND folder_id = " . $row['folder_id'];
				$db->sql_query($sql);
			}
			
		}
		
		// loop through the rowset, each row is an item in the feed
		foreach ($rowset as $row)
		{
		
			// Handle the logic to ignore a post if it does not contain the minimum number of words needed
			if (!( ($min_word_size != '') && truncate_words($row['post_text'], intval($max_word_size), true) < $min_word_size))
			{
			
				// This post goes in the newsfeed

				$username = ($row['user_id'] == ANONYMOUS) ? $row['post_username'] : $row['username'];
	
				$item = new FeedItem();
	
				// Create the title for the item (post)
				$forum_name = ($row['forum_name'] == NULL) ? $user->lang['SMARTFEED_GLOBAL_ANNOUNCEMENT'] : $row['forum_name'];
				if ($row['post_subject'] != '')
				{
					$title = ($config['sf_suppress_forum_names']) ? $row['post_subject'] : $forum_name . $user->lang['SMARTFEED_DELIMITER'] . $row['post_subject'];
				}
				else
				{
					$title = ($config['sf_suppress_forum_names']) ? 'Re: ' . $row['topic_title'] : $forum_name . $user->lang['SMARTFEED_DELIMITER'] . 'Re: ' . $row['topic_title'];
				}
				$item->title = html_entity_decode($item->title);
				
				if ($row['topic_first_post_id'] != $row['post_id'])
				{
					if ($config['sf_show_username_in_replies'])
					{
						$title .= ($row['username'] == '') ? $user->lang['SMARTFEED_DELIMITER'] . $user->lang['SMARTFEED_REPLY_BY'] . ' ' . $user->lang['GUEST'] . ' ' . $username : $user->lang['SMARTFEED_DELIMITER'] . $user->lang['SMARTFEED_REPLY_BY'] . ' ' . $username;
					}
					else
					{
						$title .= $user->lang['SMARTFEED_DELIMITER'] . $user->lang['SMARTFEED_REPLY'];
					}
				}
				else
				{
					if ($config['sf_show_username_in_first_topic_post'])
					{
						$title .= $user->lang['SMARTFEED_DELIMITER'] . $user->lang['AUTHOR'] . ' ' . $username;
					}
				}
				
				$item->title = html_entity_decode(censor_text($title));
				
				// Create a number of item attributes, not all of which are necessarily in the feed (it depends on the feed type)
				$item->date = $row['post_time'];
				$item->pubDate = $item->date;
				$item->link = $board_url . 'viewtopic.' . $phpEx . '?f=' . $row['forum_id'] . '&t=' . $row['topic_id'] . '&p=' . $row['post_id']  . '#p' . $row['post_id'];
				$item->source = $board_url;
				$item->category = html_entity_decode($row['forum_name']);
				$item->guid = $item->link;
				$item->descriptionHtmlSyndicated = true;
				$item->comments = $board_url . 'posting.' . $phpEx . '?mode=reply&f=' . $row['forum_id'] . '&t=' . $row['topic_id'];
				$item->author = $username;
	
				if ($feed_type==$config['sf_rss_20_value']) 
				{
					// RSS 2.0 requires an email field to validate. Use a fake email address (no_email@example.com, set in feed_creator.class.php 
					// if the author field is blank) unless in user profile it says it is okay to show email address.
					$item->authorEmail = (($row['user_allow_viewemail'] == '1') && (!$config['sf_privacy_mode'])) ? $row['user_email'] : '';
				}               
				else
				{
					$item->authorEmail = '';
				}
				
				// To "dress up" the post text with bbCode, images, smilies etc., we need to use generate_text_for_display() function. This is a great
				// function. Replicating this for phpBB 2 was a pain in the buttocks!
				
				$message = censor_text($row['post_text']);
				
				$user_sig = ( $row['enable_sig'] && $row['user_sig'] != '' && $config['allow_sig'] && (!($config['sf_privacy_mode']) || $registered_user) ) ? $row['user_sig'] : '';
				
				if (($feed_style == SMARTFEED_HTML) || ($feed_style == SMARTFEED_HTMLSAFE))
				{
					$flags = (($row['enable_bbcode']) ? OPTION_FLAG_BBCODE : 0) +
						(($row['enable_smilies']) ? OPTION_FLAG_SMILIES : 0) + 
						(($row['enable_magic_url']) ? OPTION_FLAG_LINKS : 0);
						
					$message = generate_text_for_display($message, $row['bbcode_uid'], $row['bbcode_bitfield'], $flags);
					
					$message = str_replace('<img src="./', '<img src="' . $board_url, $message); 
		
					if ($user_sig != '')
					{
						$user_sig = generate_text_for_display(censor_text($user_sig), $row['user_sig_bbcode_uid'], $row['user_sig_bbcode_bitfield'], $flags);
					}
		
					$message = ($user_sig != '') ? $message . $user->lang['SMARTFEED_POST_SIGNATURE_DELIMITER'] . $user_sig : $message;
					if ($feed_style == SMARTFEED_HTMLSAFE)
					{
						$message = strip_tags($message, $allowable_tags);
					}
				}
				else
				{
					// Either Compact or Basic Style wanted
					if ($feed_style == SMARTFEED_BASIC)
					{
						$message = ($user_sig != '') ? $message . "\n\n" . $user_sig : $message;
					}
					$message = strip_tags($message);	// Gets rid of any embedded HTML
					strip_bbcode($message); 			// Remove the BBCode
					// Either condense all text or make line feeds explicit
					$message = ($feed_style == SMARTFEED_BASIC) ? nl2br($message) : str_replace("\n", ' ', $message);
				}
				
				// Handle the maximum number of words requested per post logic
				if ($max_word_size != 0)
				{
					$message = truncate_words($message, intval($max_word_size));
				}
				
				// Add the item (post) to the feed
				$item->description = $message;
				$rss->addItem($item);
				
			}
			
		}
	}
	
	// Publish the feed as RSS or Atom
	$rss->outputFeed($feed_type);
	return;

}

function decrypt($encoded_64, $key)
{

	// This function decrypts $encoded_64 with the given $key. It uses the PHP Secure Communications Library, found
	// at http://phpseclib.sourceforge.net/. If mcrypt is compiled into PHP it will use that, otherwise it will use the functions 
	// from this library. This allows for maximum portability of Smartfeed. Thanks to Terrafrost at phpbb.com for the suggestion
	// to use this library! (He is the author.)

	global $phpbb_root_path, $phpEx;
	
	include($phpbb_root_path . "includes/Crypt/TripleDES.$phpEx");
	$des = new Crypt_TripleDES();

	$des->setKey($key);

	// Decrypt
	$decrypted_data = $des->decrypt(base64_decode($encoded_64));
	return $decrypted_data;
	
}  

function get_default_style()
{

	// This function returns the default style. The default style is needed to retrive the feed
	// image.

	global $config, $db;
	
	$sql = 'SELECT template_path 
			FROM ' . STYLES_TABLE . ' s JOIN ' . STYLES_TEMPLATE_TABLE . ' t ON s.template_id = t.template_id 
			WHERE style_id = ' . $config['default_style'];
	$result = $db->sql_query($sql);
	$row = $db->sql_fetchrow($result);
	return $row['template_path'];
	
}  

function truncate_words($text, $max_words, $just_count_words = false)
{

	// This function returns the first $max_words from the supplied $text. If $just_count_words === true, a word count is returned
	
	global $user;
	
	$word_array = explode(' ',$text);
	if ($just_count_words)
	{
		return sizeof($word_array);
	}
	
	if (sizeof($word_array) <= $max_words)
	{
		return rtrim($text);
	}
	else
	{
		$truncated_text = '';
		for ($i=0; $i < $max_words; $i++) 
		{
			$truncated_text .= $word_array[$i] . ' ';
		}
		return rtrim($truncated_text) . $user->lang['SMARTFEED_MAX_WORDS_NOTIFIER'];
	}
}

?>
